from bzv.infra import Infra
from bzv.customerConfig import CustomerConfig
from bzv.configWatcher import ConfigWatcher
from bzv.inbox import Inbox
from bzv.server import main
from merescocore.components import readConfig
from merescocore.components.http import ObservableHttpServer
from merescocore.framework import be
from weightless import Reactor
from sys import stdout, argv
from fcntl import flock, LOCK_EX, LOCK_NB
from os import environ, open, O_RDONLY, O_CREAT
from os.path import join as join_path

environ['LANG']='en_US.UTF-8'

import string
#import logging

args = argv[1:]
if len(args) == 1:
	configFile = args[0]
	config = readConfig(configFile)

baseURL = config['baseURL']
portNumber = int(config['portNumber'])
sruUpdatePortNumber = int(config['sruUpdatePortNumber'])
databasePath = config['databasePath']

websiteConfigfile = config['websiteConfig']
serverConfigfile = config['serverConfig']
drilldownConfigfile = config['drilldownConfig']
loggingConfigfile = config['loggingConfig']
availabilityConfigfile = config['availabilityConfig']

# make sure no two instances access the same database
lockFile = join_path(databasePath, 'lock')
fd = open(lockFile, O_RDONLY|O_CREAT)
flock(fd, LOCK_EX|LOCK_NB)

infra = Infra('Starting MerescoServer',loggingConfigfile=loggingConfigfile)

logging = infra.logging
logger = logging.getLogger('meresco-server')
logging.debug('Starting meresco-server logging')
logger.debug('Starting meresco-server logger')

websiteConfig = CustomerConfig(websiteConfigfile,infra)
serverConfig  = CustomerConfig(serverConfigfile,infra)
drilldownConfig  = CustomerConfig(drilldownConfigfile,infra)
availabilityConfig = CustomerConfig(availabilityConfigfile,infra)

configs = {'website': websiteConfig, 'server': serverConfig, 'drilldown': drilldownConfig, 'availability': availabilityConfig}

infra.configs = configs

print 'Firing up Tilburg Beter Zoeken & Vinden Server.'
reactor = Reactor()
observableHttpServer = ObservableHttpServer(reactor, portNumber)
observableHttpServer.startServer()
if portNumber < 1024:
	runAs(config['username'], config['groupname'])

sruUpdateServer = ObservableHttpServer(reactor, sruUpdatePortNumber, prio = 9)

inbox = Inbox(reactor=reactor, inboxDirectory=config['inboxDirectory'],
	      doneDirectory=config['doneDirectory'], infra = infra)

configWatcher = ConfigWatcher(reactor=reactor,
			      configWatcherDirectory = config['configWatcherDirectory'],
			      infra = infra,
			      configs=configs,
			      )

server = be(main(reactor, observableHttpServer, sruUpdateServer, inbox,
		 baseURL, portNumber, databasePath, config, infra, configWatcher))

server.once.observer_init()

inbox.processInboxDirectory()

print "Ready to rumble on", baseURL, "at ports", portNumber, sruUpdatePortNumber
print "   - database:", databasePath, "\n"
stdout.flush()
reactor.loop()
